import { Metadata, Http, Loader } from '@zeedhi/core';
import { Modal, ModalService, Form, DialogService, LoadingService, Col } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { Utilities } from '@/Services/Utilities';
import { DataSourceController } from './DataSourceController';

export class DSIndexesController {
    private mainController: DataSourceController = Loader.getInstance("DataSourceController");

    private get datasourceData() {
        return this.mainController.datasourceData;
    }

    public indexesData: any;
    private indexModal!: Modal;
    public get isAddIndex() {
        return this.indexesData === undefined;
    }
    public get isDetailIndex() {
        return this.indexesData !== undefined;
    }
    public get indexModalTitle() {
        return (this.indexesData ? "" : "Adicionar") + " Índice";
    }
    public get indexesFormGrid() {
        return this.isAddIndex ? {name: 9, column: 6} : {name: 12, column: 9};
    }
    public updateIndexesFilter() {
        // Se grid não foi montado, não faz nada
        if (Metadata.getInstances("indexes-grid").length === 0 || !Metadata.getInstances("indexes-grid")[0]) {
            return;
        }
        const dataSourceId = this.datasourceData._id;
        const indexesGrid = Metadata.getInstance<TekGrid>("indexes-grid");
        indexesGrid.datasource.setFilter({ dataSourceId });
    }
    public async openIndexesModal({ row }: any) {
        this.indexModal?.destroy();
        this.indexModal = await ModalService.createFromJson('dataSource/Indexes/IndexDetailModal', true);
        this.indexesData = row || undefined;
        this.indexModal.show();
    }
    public closeIndexesModal() {
        this.indexModal.hide();
        this.indexModal.destroy();
    }
    public indexesColumnListHolderMounted() {
        if (this.indexesData) {
            const columnList = Metadata.getInstance<Col>("indexesColumnList");
            const indexesForm = Metadata.getInstance<Form>("indexesForm");
            const {name, key} = this.indexesData;
            indexesForm.value.indexName = name;
            Object.keys(key).forEach((column: string) => {
                this.newColumnOnIndexForm({column, order: key[column] === 1 ? "Crescente" : "Decrescente"});
            });
            // Remove primeiro elemento que vem como padrão
            columnList.childrenInstances.splice(0, 1);
            columnList.children.splice(0, 1);
        }
    }
    public async columnListOnIndexFormMounted({ component }: any) {
        const row = component.childrenInstances[0];
        const val = component.children[0].value;
        await row.datasource.updateData(this.datasourceData.columns);
        if (val) {
            row.setValue(val);
        }
    }
    public newColumnOnIndexForm(data: any) {
        const columnList = Metadata.getInstance<Col>("indexesColumnList");
        const usedNumbers = columnList.children.map((params: any) => Number(params.name.split('_')[1])).sort((a: number, b: number) => (a > b ? 1 : -1));
        let newNumber = usedNumbers.length;
        for (let i = 0; i < usedNumbers.length; i += 1) {
            if (usedNumbers[i] !== i) {
                newNumber = i;
                break;
            }
        }
        const template = JSON.parse(JSON.stringify({...columnList.children[0]}));
        template.name = "columnRow_" + newNumber;
        template.children = template.children?.map((obj: any) => ({...obj, value: "", name: obj.name.split('_')[0] + '_' + newNumber })) || [];
        if (data && data.column && data.order) {
            template.children[0].value = data.column;
            template.children[1].value = data.order;
        } else {
            template.children[1].value = "Crescente";
        }
        columnList.children.push(template);
        return template;
    }
    public removeColumnOnIndexForm({ component }: any) {
        const columnList = Metadata.getInstance<Col>("indexesColumnList");
        const column = component.parent;
        const index = columnList.childrenInstances.findIndex((el: any) => el.name === column.name);
        if (index !== -1 && columnList.children.length > 1) {
            columnList.childrenInstances.splice(index, 1);
            columnList.children.splice(index, 1);
        }
    }
    public confirmSaveIndexes() {
        const indexesForm = Metadata.getInstance<Form>("indexesForm");
        const formsHolder = Metadata.getInstance<Col>("indexesColumnList");
        //  A utilização do map permite que o metódo validate seja chamado em todos os formulários, diferente do every que pararia no primeiro false
        if (![indexesForm, ...formsHolder.childrenInstances].map((form: any) => form.validate()).every((valid: boolean) => valid)) {
            return;
        }
        DialogService.show({
            name: "indexSaveConfirm",
            type: "warning",
            title: "Confirmação",
            text: "Em fontes de dados muito extensas, o processo de criação de índices pode demorar muito tempo. A fonte de dados ficará inacessível até o término deste processo e não será possível interrompê-lo.<br><br>Deseja continuar?",
            buttons: [
                { name: 'não', component: 'ZdButton', label: 'Não', outline: true, events: { click: () => DialogService.hide() } },
                { name: 'sim', component: 'ZdButton', label: 'Sim', events: { click: () => { this.saveIndexesForm(); DialogService.hide(); } } }
            ],
        });
    }
    public async saveIndexesForm() {
        const indexesForm = Metadata.getInstance<Form>("indexesForm");
        const name = indexesForm.value.indexName;
        const formsHolder = Metadata.getInstance<Col>("indexesColumnList");
        const columnsValue = formsHolder.childrenInstances.map((params: any) => {
            const number = Number(params.name.split('_')[1]);
            return { direction: params.value["order_" + number] === "Crescente" ? 1 : -1, name: params.value["column_" + number]};
        });
        const data = {
            name,
            dataSource: this.datasourceData._id,
            COLUMN_: columnsValue.map((column: any) => column.name),
            ORDER_: columnsValue.map((column: any) => column.direction)
        };
        try {
            LoadingService.show();
            const res = await Http.post("/dataSource/saveIndexes", {
                requestType: "Row",
                row: data
            });
            LoadingService.hide();
            if (res.error) {
                CustomAlertService.errorAlert(res.error);
            } else {
                CustomAlertService.saveSuccessAlert();
                this.closeIndexesModal();
                this.reloadIndexesGrid();
            }
        } catch (e) {
            LoadingService.hide();
            CustomAlertService.errorAlert(String(e));
            console.error(e);
        }
    }
    public async deleteIndexes(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/dataSource/deleteIndexes", "indexes-grid", "Row");
    }
    public reloadIndexesGrid() {
        const grid = Metadata.getInstance<TekGrid>("indexes-grid");
        grid.reload();
    }
}
